use futures::{SinkExt, StreamExt};
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};
use crate::utils;

pub(crate) async fn handle_client(stream: tokio::net::TcpStream) -> Result<(), Error> {
    let must_existed_params = vec!["deviceId", "fromId", "wsPwd"];
    let mut ws_stream = accept_hdr_async(stream, |req: &Request, resp: Response| {
        println!("新客户端连接: {}", req.uri());
        let connection_params = match utils::get_connection_params(req.uri().to_string()) {
            Ok(p) => p,
            Err(e) => {
                println!("缺少重要连接数据段: {}", e);
                return Err(Error::ConnectionClosed);
            }
        };
        let not_existed = must_existed_params.find(|param| !connection_params.contains_key(param));
        if not_existed != None {
            println!("缺少重要连接数据段: {}", not_existed);
            return Err(Error::ConnectionClosed);
        }
        Ok(resp)
    })
        .await
        .expect("WebSocket handshake error");
    while let Some(msg) = ws_stream.next().await {
        let msg = msg?;
        if msg.is_text() {
            println!("Client message: {}", msg.to_text()?);
            ws_stream
                .send(Message::text(format!("Your message is {}", msg)))
                .await?;
        }
    }
    println!("Client finished");
    Ok(())
}
